//////////////////////////////////////////////////////////////////////////////////////
//  
//  FMPluginHighlight.cpp - Sample external function for FileMaker plug-in
//
//  Copyright 2003-2004 by 24U Software. All rights reserved.
//
//  Written by Tomas Zahradnicky & HOnza Koudelka
//
////////////////////////////////////////////////////////////////////////////////
//
//  This example is based on 24U FM Template, a stationery for building
//  FileMaker plug-ins from 24u Software.
//
//  The latest version of 24U FM Template is available for download from:
//
//  http://24uSoftware.com/FMTemplate  or  http://www.NewMillennium.com/
//
////////////////////////////////////////////////////////////////////////////////
//
//  24U Sample Code - Terms of use
//
//  You may incorporate this sample code into your products without
//  restriction, though the code has been provided "AS IS" and the
//  responsibility for its operation is 100% yours.
//
//  You are not however permitted to redistribute the source
//  as "24U Sample Code" after having made changes.
//
//  If you are going to redistribute the source with references to
//  24U Software, we require that you make it clear in the source that
//  the code was descended from 24U Sample Code, but that you've made changes.
//
////////////////////////////////////////////////////////////////////////

//
//  This function takes two textual parameters. The first is a search string
//  and the second is a text we want to search within. The result is the same
//  text, but with all occurrences of the search string highlighted by applying
//  color and style which can be set up in the plug-in's preferences.
//
////////////////////////////////////////////////////////////////////////////////


#include "FMPlugin.h"
#include "FMTemplate/FMTemplate.h"
#include "FMPluginPreferences/FMPluginPreferences.h"

#pragma mark Plugin_Highlight
FMX_PROC(fmx::errcode) Plugin_Highlight(short funcId, const fmx::ExprEnv& environment, const fmx::DataVect& dataVect, fmx::Data& result)
{
    #pragma unused(funcId,environment)                                          // This function is supposed to work anywhere

    fmx::errcode                errorResult = 0;                                // Start with no error
    fmx::CharacterStyleAutoPtr  style;
    fmx::ColorAutoPtr           color;
    fmx::ulong                  position;
    fmx::CharacterStyle::Face   face = 0;                                       // Base style is plain text, we'll modify it later
    fmx::TextAutoPtr            hilitedResult;

    const fmx::Text&	        findWhat  = dataVect.AtAsText(0);               // Extract parameters passed to the function
    const fmx::Text&            hiliteWhere = dataVect.AtAsText(1);
    const PluginPreferences*    prefs = GetPreferences();                       // Get color and face to use for highlighting
  
    (*hilitedResult).InsertText( hiliteWhere, 0 );                              // Copy the text to search within
    
    if( findWhat.GetSize()>0 )                                                  // Don't do anything if there is nothing to find
    {
        (*color).SetRed(prefs->fColor.red>>8);                                  // Prepare a color object
        (*color).SetGreen(prefs->fColor.green>>8);                              // from the color set in the
        (*color).SetBlue(prefs->fColor.blue>>8);                                // plug-in's preferences

        if( prefs->fBold )      face |= fmx::CharacterStyle::kFace_Bold;        // Prepare a face object
        if( prefs->fUnderline ) face |= fmx::CharacterStyle::kFace_Underline;   // from the face combination
        if( prefs->fItalic )    face |= fmx::CharacterStyle::kFace_Italic;      // set in the plug-in's preferences
        
        for(   position  = hiliteWhere.Find( findWhat, 0);                      // Find the first occurrence of the search string
               position != fmx::Text::kSize_End;                                // and then repeat the following block for all
               position  = hiliteWhere.Find( findWhat, position+1) )            // occurrences
        {
            (*hilitedResult).GetStyle( *style, position );                      // Preserve the original style
            (*style).SetFace((*style).GetFace() | face);                        // Apply highlight face, preserving original face
            (*style).SetColor(*color);                                          // Apply highlight color
   
            (*hilitedResult).SetStyle(*style, position, findWhat.GetSize());    // Apply the modified style to the found text
        }
    }

    result.SetAsText (*hilitedResult, dataVect.At(0).GetLocale() );             // Copy the modified text to the result
    
    return(errorResult);
}