//////////////////////////////////////////////////////////////////////////////////////
//  
//  FMPluginPreferencesMac.cpp - Sample support file for FileMaker plug-in
//
//  Copyright 2003-2004 by 24U Software. All rights reserved.
//
//  Written by Tomas Zahradnicky & HOnza Koudelka
//
////////////////////////////////////////////////////////////////////////////////
//
//  This example is intended for a FileMaker plug-in based on 24U FM Template,
//  a stationery from 24u Software.
//
//  The latest version of 24U FM Template is available for download from:
//
//  http://24uSoftware.com/FMTemplate  or  http://www.NewMillennium.com/
//
////////////////////////////////////////////////////////////////////////////////
//
//  24U Sample Code - Terms of use
//
//  You may incorporate this sample code into your products without
//  restriction, though the code has been provided "AS IS" and the
//  responsibility for its operation is 100% yours.
//
//  You are not however permitted to redistribute the source
//  as "24U Sample Code" after having made changes.
//
//  If you are going to redistribute the source with references to
//  24U Software, we require that you make it clear in the source that
//  the code was descended from 24U Sample Code, but that you've made changes.
//
////////////////////////////////////////////////////////////////////////

//
//  This file provides support for displaying a preferences dialog
//  after the user clicks on the Configure button in the application
//  preferences of FileMaker Pro.
//
////////////////////////////////////////////////////////////////////////////////


#include <Carbon/Carbon.h>
#include <CoreServices/CoreServices.h>

#include "FMTConfig.h"
#include "FMTemplate/FMTemplate.h"
#include "ColorSwatchMac/ColorSwatch.h"
#include "FMPluginPreferences/FMPluginPreferences.h"

enum {
    kControlProxyID     = 1000,
    kControlBoldID      = 1001,
    kControlUnderlineID = 1002,
    kControlItalicID    = 1003,
    kControlSwatchID    = 1004
};

#pragma mark gPreferences

PluginPreferences   gPreferences={{0xFFFF, 0, 0}, 1, 0, 0};     // Make Bold Red the default

#pragma mark -

static Boolean PRReadPreferences(SInt16 iFile, PluginPreferences* ioPrefs)
{
	OSStatus 	        err = 0;
	PluginPreferences   prefs = {0,};
	
	err = FSReadFork(iFile,fsFromStart,0LL,sizeof(PluginPreferences),&prefs,NULL);
	
	if (err == noErr) *ioPrefs = prefs;

	return err == noErr;
}

static Boolean PRWritePreferences(SInt16 iFile, const PluginPreferences* iPrefs)
{
	OSStatus 	err = FALSE;

	err = FSWriteFork(iFile,fsFromStart,0LL,sizeof(PluginPreferences),iPrefs,NULL);
	
	return err == noErr;
}

static OSStatus PRSetWindowPreferences( WindowRef iWindow, const PluginPreferences* iPrefs )
{
    ControlID   cid        = {0};
    OSStatus    err        = noErr;
    RGBColor    color;
    ControlRef  cSwatch    = NULL, 
                cBold      = NULL, 
                cUnderline = NULL, 
                cItalic    = NULL;
    
    cid.id = kControlSwatchID;
    err = GetControlByID( iWindow, &cid, &cSwatch );
    
    if( err == noErr ) 
    {
        cid.id = kControlBoldID;
        err = GetControlByID( iWindow, &cid, &cBold );
    }

    if( err == noErr ) 
    {
        cid.id = kControlUnderlineID;
        err = GetControlByID( iWindow, &cid, &cUnderline );
    }

    if( err == noErr ) 
    {
        cid.id = kControlItalicID;
        err = GetControlByID( iWindow, &cid, &cItalic );
    }
    
    if( err == noErr )
    {
        color.red = iPrefs->fColor.red;
        color.green = iPrefs->fColor.green;
        color.blue = iPrefs->fColor.blue;

        err = HIColorSwatchSetColor( cSwatch, color );
    }
    
    if( err == noErr )
    {
        SetControlValue( cBold, iPrefs->fBold);
        SetControlValue( cUnderline, iPrefs->fUnderline );
        SetControlValue( cItalic, iPrefs->fItalic );
    }
    
    return err;
}            


    

static OSStatus PRGetWindowPreferences( WindowRef iWindow, PluginPreferences* ioPrefs )
{
    ControlID   cid        = {0};
    OSStatus    err        = noErr;
    RGBColor    color;
    ControlRef  cSwatch    = NULL, 
                cBold      = NULL, 
                cUnderline = NULL, 
                cItalic    = NULL;
    
    cid.id = kControlSwatchID;
    err = GetControlByID( iWindow, &cid, &cSwatch );
    
    if( err == noErr ) 
    {
        cid.id = kControlBoldID;
        err = GetControlByID( iWindow, &cid, &cBold );
    }

    if( err == noErr ) 
    {
        cid.id = kControlUnderlineID;
        err = GetControlByID( iWindow, &cid, &cUnderline );
    }

    if( err == noErr ) 
    {
        cid.id = kControlItalicID;
        err = GetControlByID( iWindow, &cid, &cItalic );
    }
    
    if( err == noErr )
        err = HIColorSwatchGetColor( cSwatch, &color );

    if( err == noErr )
    {
        ioPrefs->fColor.red = color.red;
        ioPrefs->fColor.green = color.green;
        ioPrefs->fColor.blue = color.blue;
        ioPrefs->fBold = GetControlValue( cBold );
        ioPrefs->fUnderline = GetControlValue( cUnderline );
        ioPrefs->fItalic = GetControlValue( cItalic );
    }
    
    return err;
}            


static pascal OSStatus PRSwatchChangedHandler( EventHandlerCallRef inHandlerCallRef, EventRef inEvent, void *inUserData )
{
    #pragma unused( inHandlerCallRef )

    OSStatus    err = noErr;
    HICommand   cmd;
    ControlRef  swatch = *(ControlRef*)inUserData;
    
    err = GetEventParameter( inEvent, kEventParamDirectObject, typeHICommand, NULL, sizeof(cmd), NULL, &cmd );
    
    if( err == noErr )
    {
        switch( cmd.commandID )
        {

            case    kControlSwatchID:
                DrawOneControl( swatch );
                break;
    
            default:
                err = eventNotHandledErr;
                break;
        }
    }
    
    return err;
}

static pascal OSStatus PRWindowHandler( EventHandlerCallRef inHandlerCallRef, EventRef inEvent, void *inUserData )
{
    #pragma unused( inHandlerCallRef )

    OSStatus    err = noErr;
    HICommand   cmd;
    WindowRef   window = *(WindowRef*)inUserData;
    
    err = GetEventParameter( inEvent, kEventParamDirectObject, typeHICommand, NULL, sizeof(cmd), NULL, &cmd );
    
    if( err == noErr )
    {
        switch( cmd.commandID )
        {
            case    kHICommandOK:
                PRGetWindowPreferences(window,&gPreferences);
                //  fall thru
            case    kHICommandCancel:
                QuitAppModalLoopForWindow(window);
                break;
        
            default:    
                err = eventNotHandledErr;
                break;
        }
    }    
    return err;
}

static OSStatus PRGetFileNameAsUnicode(const char* iFileName, HFSUniStr255* oFileName)
{
    TextToUnicodeInfo   ttu = NULL;
    OSStatus            err = noErr;
    TextEncoding        enc;
    ByteCount           sr = 0;
    ByteCount           len = 0;
    
    err = UpgradeScriptInfoToTextEncoding(smRoman,kTextLanguageDontCare,kTextRegionDontCare,NULL,&enc);
    
    if( err == noErr )
        err = CreateTextToUnicodeInfoByEncoding(enc, &ttu);

    if( err == noErr )
        err = ConvertFromTextToUnicode(ttu,strlen(iFileName),iFileName,kNilOptions,0,NULL,NULL,NULL,sizeof(oFileName->unicode),&sr,&len,oFileName->unicode);
    
    if( err == noErr )
        oFileName->length = sr;
        
    if( ttu )
        DisposeTextToUnicodeInfo(&ttu);    
        
        
    return err;
}

static void PRGetDefaultPreferences(PluginPreferences* oPrefs)
{
	oPrefs->fColor.red = 0xFFFF;
	oPrefs->fColor.green = 0;
	oPrefs->fColor.blue = 0;
	oPrefs->fBold = TRUE;
	oPrefs->fUnderline = FALSE;
	oPrefs->fItalic = FALSE;
}

static SInt16 PRGetPrefsFile(Boolean iWantWriteAccess)
{
	SInt16 		        forkRef=0;
	FSRef               prefsFolderRef;
	FSRef               prefsFileRef;
	HFSUniStr255        prefsFileName;
	PluginPreferences	prefs;
	OSStatus			err = 0;
    TextEncoding        enc;
	
    err = FSFindFolder(kUserDomain,kPreferencesFolderType,TRUE,&prefsFolderRef);
	
	if( err == noErr )
	    err = PRGetFileNameAsUnicode(PLUGIN_PREFERENCES_FILE_NAME,&prefsFileName);
	    
	if( err == noErr )
	{	
		err = FSCreateFileUnicode( &prefsFolderRef, prefsFileName.length, prefsFileName.unicode, kFSCatInfoNone, NULL, &prefsFileRef, NULL); 

        if( err == dupFNErr )
        {
            err = UpgradeScriptInfoToTextEncoding(smRoman,kTextLanguageDontCare,kTextRegionDontCare,NULL,&enc);
            if (err == noErr)
                err = FSMakeFSRefUnicode(&prefsFolderRef, prefsFileName.length, prefsFileName.unicode, kTextEncodingUnknown, &prefsFileRef);
            if (err == noErr)
                err = FSOpenFork(&prefsFileRef,0,NULL,(iWantWriteAccess?fsRdWrPerm:fsRdPerm),&forkRef);
        }
        else if( err == noErr )
        {
            err = FSOpenFork(&prefsFileRef,0,NULL,fsRdWrPerm,&forkRef);
            if( err == noErr )
			{
    			PRGetDefaultPreferences(&prefs);
    			PRWritePreferences(forkRef,&prefs);
    			FSCloseFork(forkRef);		
                err = FSOpenFork(&prefsFileRef,0,NULL,(iWantWriteAccess?fsRdWrPerm:fsRdPerm),&forkRef); 
            }
        }
   	}
	
	if (err == noErr)
	    return forkRef;
	return 0;
}

DEFINE_ONE_SHOT_HANDLER_GETTER(PRWindowHandler);
DEFINE_ONE_SHOT_HANDLER_GETTER(PRSwatchChangedHandler);

#pragma mark -

void ReadPreferences()
{
	SInt16 	forkRef = PRGetPrefsFile(FALSE);
	Boolean	result = FALSE;
	
	if( forkRef )
	{
		result = PRReadPreferences(forkRef, &gPreferences);
	
		FSCloseFork(forkRef);
	}
}

void DisplayPreferences()
{
    IBNibRef            nib =  NULL;
    OSStatus            err =  noErr;
    WindowRef           window = NULL;
    ControlID           cid={0};
    HIRect              rect;
    CGrafPtr            savedPort = NULL;
    ControlRef          cBold=NULL, 
                        cUnderline=NULL, 
                        cItalic=NULL, 
                        cSwatch=NULL;
    RGBColor            color;
    EventTypeSpec       events[] = {{ kEventClassCommand, kEventCommandProcess }};
    EventHandlerRef     handler = NULL;
                       
	SInt16 	            prefsFileRef = PRGetPrefsFile(TRUE);	

    GetPort(&savedPort);
    
    err = CreateNibReferenceWithCFBundle( GetPluginBundle(), CFSTR("FMPluginPreferences"), &nib );
    
    if( err == noErr )
        err = CreateWindowFromNib( nib, CFSTR("FMPluginPreferences"), &window );
    
    if( err == noErr )
    {
        SetPortWindowPort( window );

        cid.id=kControlProxyID;
        err = GetControlByID( window, &cid, &cSwatch );    
    }
    
    if( err == noErr )
        err = HIViewGetFrame( cSwatch, &rect );

    if( err == noErr )
    {
        color.red   = gPreferences.fColor.red;
        color.green = gPreferences.fColor.green;
        color.blue  = gPreferences.fColor.blue;
        
        DisposeControl(cSwatch);
        err = HIColorSwatchCreate( window, &rect, color, &cSwatch );
    }    
    
    if( err == noErr )
    {
        HIViewSetVisible( cSwatch, true );
        
        cid.id = kControlSwatchID;
        SetControlID( cSwatch, &cid );
        SetControlCommandID( cSwatch, kControlSwatchID );
    }
    
    if( err == noErr )
        err = PRSetWindowPreferences( window, &gPreferences );
    
    if( err == noErr )
        err = InstallWindowEventHandler( window, GetPRWindowHandlerUPP(), GetEventTypeCount(events), events, &window, NULL );

    if( err == noErr )
        err = InstallApplicationEventHandler( GetPRSwatchChangedHandlerUPP(), GetEventTypeCount(events), events, &cSwatch, &handler );

    if( err == noErr )
    {        
        TransitionWindow( window, kWindowZoomTransitionEffect, kWindowShowTransitionAction, NULL );
        RunAppModalLoopForWindow( window );
        TransitionWindow( window, kWindowZoomTransitionEffect, kWindowHideTransitionAction, NULL );
    }
    
    MacSetPort( savedPort );

	//  Write updated preferences to disk
	if( prefsFileRef )
	{
		(void) PRWritePreferences(prefsFileRef, &gPreferences);
		FSCloseFork(prefsFileRef);
	}

    if( handler )
        RemoveEventHandler( handler );

    if( window )
        DisposeWindow( window );
    
    if( nib )
        DisposeNibReference( nib );  
}

const PluginPreferences* GetPreferences()
{
    return &gPreferences;
}
