//////////////////////////////////////////////////////////////////////////////////////
//  
//  FMPluginPreferencesWin.cpp - Sample support file for FileMaker plug-in
//
//  Copyright 2003-2004 by 24U Software. All rights reserved.
//
//  Written by Tomas Zahradnicky & HOnza Koudelka
//
////////////////////////////////////////////////////////////////////////////////
//
//  This example is intended for a FileMaker plug-in based on 24U FM Template,
//  a stationery from 24u Software.
//
//  The latest version of 24U FM Template is available for download from:
//
//  http://24uSoftware.com/FMTemplate  or  http://www.NewMillennium.com/
//
////////////////////////////////////////////////////////////////////////////////
//
//  24U Sample Code - Terms of use
//
//  You may incorporate this sample code into your products without
//  restriction, though the code has been provided "AS IS" and the
//  responsibility for its operation is 100% yours.
//
//  You are not however permitted to redistribute the source
//  as "24U Sample Code" after having made changes.
//
//  If you are going to redistribute the source with references to
//  24U Software, we require that you make it clear in the source that
//  the code was descended from 24U Sample Code, but that you've made changes.
//
////////////////////////////////////////////////////////////////////////

//
//  This file provides support for displaying a preferences dialog
//  after the user clicks on the Configure button in the application
//  preferences of FileMaker Pro.
//
////////////////////////////////////////////////////////////////////////////////


#include <shlobj.h>
#include "FMTConfig.h"
#include "FMTemplate/FMTemplate.h"
#include "ColorSwatchPC/ColorSwatchPC.h"
#include "FMPluginPreferences/FMPluginPreferences.h"

enum {
    kControlProxyID     = 1000,
    kControlBoldID      = 1001,
    kControlUnderlineID = 1002,
    kControlItalicID    = 1003,
    kControlSwatchID    = 1004
};

#pragma mark gPreferences

PluginPreferences   gPreferences={{0xFFFF, 0, 0}, 1, 0, 0};     // Make Bold Red the default

#define MKCOLOR(x)  RGB((x).red >> 8, (x).green >> 8, (x).blue >> 8)

#pragma mark -

static void PRStrCatW(wchar_t* ioPath, const char* iPath)
{
	int result;
	wchar_t	buffer[1024];
	
	memset(buffer,0,sizeof(buffer));
	
	result = MultiByteToWideChar(CP_ACP, 0, iPath, -1, buffer, sizeof(buffer)/sizeof(wchar_t));

	if( result )
	{
		lstrcatW(ioPath,buffer);
	}
	
}

static BOOL PRGetPrefsFolder(wchar_t ioPath[MAX_PATH])
{
	BOOL result = 0;
	
	static HRESULT STDAPICALLTYPE (* _SHGetFolderPathW)(HWND, int, HANDLE, DWORD, LPWSTR) = NULL;

	HMODULE 	module;
	
	if( !_SHGetFolderPathW )
	{
		module = GetModuleHandle("Shell32.dll");
	
		_SHGetFolderPathW = (HRESULT STDAPICALLTYPE (*)(HWND,int,HANDLE,DWORD,LPWSTR))GetProcAddress(module,"SHGetFolderPathW");
	}
	
	if( _SHGetFolderPathW )
	{
		result = _SHGetFolderPathW(NULL, CSIDL_APPDATA|CSIDL_FLAG_CREATE, NULL, 0, ioPath);
		
		if(SUCCEEDED(result))
		{
			PRStrCatW(ioPath,"\\" COMPANY_NAME);
			result = CreateDirectoryW(ioPath,NULL);
		}

		if(SUCCEEDED(result))
		{
			PRStrCatW(ioPath,"\\" PLUGIN_NAME);
			result = CreateDirectoryW(ioPath,NULL);
		}

		if(SUCCEEDED(result))
			PRStrCatW(ioPath,"\\");

	}	
	return result;
}

static void PRGetDefaultPreferences(PluginPreferences* oPrefs)
{
	oPrefs->fColor.red = 0xFFFF;
	oPrefs->fColor.green = 0;
	oPrefs->fColor.blue = 0;
	oPrefs->fBold = TRUE;
	oPrefs->fUnderline = FALSE;
	oPrefs->fItalic = FALSE;
}

static BOOL PRReadPreferences(HANDLE iFile, PluginPreferences* ioPrefs)
{
	BOOL 	            success = FALSE;
	int 	            err = 0;
	DWORD	            ignore = 0;
	PluginPreferences   prefs = {0,};
	
	success = ReadFile(iFile,&prefs,sizeof(PluginPreferences),&ignore,NULL);
	
	if (success) *ioPrefs = prefs;

	return success;
}

static BOOL PRWritePreferences(HANDLE iFile, const PluginPreferences* iPrefs)
{
	BOOL 	success = FALSE;
	DWORD	ignore = sizeof(PluginPreferences);

	success = WriteFile(iFile,iPrefs,ignore,&ignore,NULL);
	
	return success;
}


static HANDLE PRGetPrefsFile(BOOL iWantWriteAccess)
{
	HANDLE 		        h = INVALID_HANDLE_VALUE;
	wchar_t 	        path[MAX_PATH];
	BOOL 		        success;
	PluginPreferences	prefs;
	INT			        err = 0;
	
	success = PRGetPrefsFolder(path);
	
	if( SUCCEEDED(success) )
	{	
		PRStrCatW(path,PLUGIN_PREFERENCES_FILE_NAME);
		
		h = CreateFileW(path, GENERIC_READ | (iWantWriteAccess?GENERIC_WRITE:0), 0, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
		err = GetLastError();
		if( (h == INVALID_HANDLE_VALUE) && (ERROR_FILE_NOT_FOUND == err) )
		{
			//  prefs file does not exist, create one
			h = CreateFileW(path, GENERIC_READ | GENERIC_WRITE, 0, NULL, CREATE_NEW, FILE_ATTRIBUTE_NORMAL, NULL );
			
			if( h != INVALID_HANDLE_VALUE )
			{
				PRGetDefaultPreferences(&prefs);
				PRWritePreferences(h,&prefs);
				CloseHandle(h);		
				h = CreateFileW(path, GENERIC_READ | (iWantWriteAccess?GENERIC_WRITE:0), 0, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL );
			}
		}
	}
	
	if( h == INVALID_HANDLE_VALUE )
		h = NULL;
	
	return h;
}

static BOOL PRLoadDialog(HWND iDialog)
{
	HWND        swatch,bold,underline,italic;
	
	swatch = GetDlgItem(iDialog,3);
	bold = GetDlgItem(iDialog,4);
	underline = GetDlgItem(iDialog,5);
	italic = GetDlgItem(iDialog,6);
	
	ColorSwatchMakeFrom(swatch,MKCOLOR(gPreferences.fColor));
	
	if(gPreferences.fBold) 
		SendMessage(bold, BM_SETCHECK, BST_CHECKED, 0);

	if(gPreferences.fUnderline) 
		SendMessage(underline, BM_SETCHECK, BST_CHECKED, 0);

	if(gPreferences.fItalic) 
		SendMessage(italic, BM_SETCHECK, BST_CHECKED, 0);

	return TRUE;
}

static void PRSaveDialog(HWND iDialog)
{
	HWND        swatch,bold,underline,italic;
	COLORREF    color;
	
	swatch = GetDlgItem(iDialog,3);
	bold = GetDlgItem(iDialog,4);
	underline = GetDlgItem(iDialog,5);
	italic = GetDlgItem(iDialog,6);
	
	color = ColorSwatchGetColor(swatch);
	gPreferences.fColor.red = GetRValue(color) << 8;
	gPreferences.fColor.green = GetGValue(color) << 8;
	gPreferences.fColor.blue = GetBValue(color) << 8;
	gPreferences.fBold = (SendMessage(bold, BM_GETCHECK, 0, 0) & BST_CHECKED)!=0;
	gPreferences.fUnderline = (SendMessage(underline, BM_GETCHECK, 0, 0) & BST_CHECKED)!=0;
	gPreferences.fItalic = (SendMessage(italic, BM_GETCHECK, 0, 0) & BST_CHECKED)!=0;
}


static BOOL CALLBACK PRDialogProc(
  HWND hwndDlg,  // handle to dialog box
  UINT uMsg,     // message
  WPARAM wParam, // first message parameter
  LPARAM lParam  // second message parameter
)
{
	BOOL result = FALSE;
    LPDRAWITEMSTRUCT lpdis;
	
	switch(uMsg)
	{
		case	WM_INITDIALOG:
			result = PRLoadDialog(hwndDlg);
			break;
		
        case 	WM_DRAWITEM: 
            lpdis = (LPDRAWITEMSTRUCT) lParam; 
            ColorSwatchDraw(lpdis);
            result = TRUE; 
 			break;
 			
        case WM_COMMAND: 
            if (wParam == IDOK)
            {
            	PRSaveDialog(hwndDlg);
                EndDialog(hwndDlg, TRUE); 
                result = TRUE; 
            }
            else if( wParam == IDCANCEL )
            {
                EndDialog(hwndDlg, TRUE); 
                result = TRUE; 
            } 
            if (HIWORD(wParam) == BN_CLICKED) 
            { 
            	if(LOWORD(wParam) == 3)
            	{
            	    COLORREF    color;
            		
            		ColorSwatchClick((HWND)lParam);
            		
            		color = ColorSwatchGetColor((HWND)lParam);
            		
                  gPreferences.fColor.red = GetRValue(color) << 8;
                	gPreferences.fColor.green = GetGValue(color) << 8;
                	gPreferences.fColor.blue = GetBValue(color) << 8;
            	
            		result = TRUE;
            	}
            } 
            break; 
	
	}

	return result;
}

#pragma mark -

void ReadPreferences()
{
	HANDLE 	h = PRGetPrefsFile(FALSE);
	BOOL	result = FALSE;
	
	if( h )
	{
		result = PRReadPreferences(h, &gPreferences);
	
		CloseHandle(h);
	}
}

void DisplayPreferences()
{
	HANDLE 	h = PRGetPrefsFile(TRUE);
	BOOL	result = FALSE;
	
	//  Display preferences dialog
	DialogBox(GetPluginInstance(),MAKEINTRESOURCE(PLUGIN_PREFERENCES_DIALOG_ID), NULL, PRDialogProc);
	
	//  Write updated preferences to disk
	if( h )
	{
		result = PRWritePreferences(h, &gPreferences);
		CloseHandle(h);
	}
	
	if (!result)
	    MessageBoxA(NULL, "Preferences could not be saved to disk.", (char*)"\0\0", MB_OK|MB_ICONEXCLAMATION);
}

const PluginPreferences* GetPreferences()
{
    return &gPreferences;
}
